/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server;

import java.io.Serializable;
import java.util.Iterator;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.quartz.JobExecutionContext;
import org.quartz.StatefulJob;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.util.MethodInvoker;

/**
 * <i>Quartz</i> job for scheduled updates of the content of the cache.
 * It is stateful to avoid expensive concurrent updates.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class ServiceResultUpdaterJob extends QuartzJobBean implements StatefulJob {// TODO vagy CacheUpdaterJob
	private final static Log log = LogFactory.getLog(ServiceResultUpdaterJob.class);

	/** <i>EHCache</i> implementation. */
	private Cache cache;

	public void setCache(Cache cache) {
		this.cache = cache;
	}

	/** Updates the content of the cache, triggered by an external scheduler. */
	@Override
	protected void executeInternal(JobExecutionContext executionContext) {
		log.info("Running scheduled update job...");

		// iterate through the current content of the cache
		for(Iterator it = cache.getKeys().iterator(); it.hasNext();) {
			Serializable key = (Serializable)it.next();
			log.info(String.format("Updating \"%s\"...", key));

			// reinvoke the service method based on the saved information
			try {
				UpdateableCachedValue value = (UpdateableCachedValue)cache.get(key).getValue();

				MethodInvoker invoker = new MethodInvoker();
				invoker.setTargetObject(value.getInvocation().getThis());
				invoker.setTargetMethod(value.getInvocation().getMethod().getName());
				invoker.setArguments(value.getInvocation().getArguments());
				invoker.prepare();
				Serializable methodResult = (Serializable)invoker.invoke();

				// update cache
				if(methodResult != null) {
					// save the unmodified invocation information along with the method result (thread safe)
					cache.remove(key);
					cache.put(new Element(key, new UpdateableCachedValue(value.getInvocation(), methodResult)));
					log.debug(String.format("\"%s\" updated in cache", key));
				}
			} catch(Exception ex) {
				log.error("Unable to reinvoke service method", ex);
			}
		}

		log.info("Update job completed.");
	}
}

//DOC: ez fuggetlen a user requestek-tol es nincs is szinkronizalva, igy lehet h egyszerre tobb frissites tortenik! (nem kivedheto, hiszen cancel is tortenhet)
//DOC: service POJO-knak thread safe-nek kell lennie vagy ugy szinkronizalni, h 2 ne mehessen egyszerre, de a user-e legyen az elsobbseg
